@@ -5,6 +5,7 @@
 import sys
 import argparse
 import structlog
+import json
 
 from . import core
 
@@ -69,10 +70,15 @@ def take() -> None:
 def cmp() -> None:
   parser = argparse.ArgumentParser(description='compare version records of nvchecker')
   core.add_common_arguments(parser)
+  parser.add_argument('-j', '--json', action='store_true',
+                      help='Output JSON array of dictionaries with {name, newver, oldver, [delta]} '
+                           '(or array of names if --quiet)')
   parser.add_argument('-q', '--quiet', action='store_true',
                       help="Quiet mode, output only the names.")
+  parser.add_argument('-a', '--all', action='store_true',
+                      help="Include unchanged versions.")
   parser.add_argument('-s', '--sort',
-                      choices=('parse_version', 'vercmp'), default='parse_version',
+                      choices=('parse_version', 'vercmp', 'none'), default='parse_version',
                       help='Version compare method to backwards the arrow '
                            '(default: parse_version)')
   parser.add_argument('-n', '--newer', action='store_true',
@@ -94,22 +100,85 @@ def cmp() -> None:
 
   oldvers = core.read_verfile(oldverf)
   newvers = core.read_verfile(newverf)
-  for name, newver in sorted(newvers.items()):
+
+  differences = []
+
+  for name, newver in sorted(newvers.items()):  # accumulate differences
     oldver = oldvers.get(name, None)
-    if oldver != newver:
-      if args.quiet:
-        print(name)
+
+    diff = {
+      'name': name,
+      'oldver': oldver,
+      'newver': newver
+    }
+
+    if oldver is not None and newver is not None:
+      if oldver == newver:
+        diff['delta'] = 'equal'
+
+      elif args.sort == "none":
+        diff['delta'] = 'new'  # assume it's a new version if we're not comparing
+
       else:
-        from .lib.nicelogger import Colors, support_color
-        c = Colors(support_color(sys.stdout))
+        from .sortversion import sort_version_keys
+        version = sort_version_keys[args.sort]
 
-        arrow = "->"
-        if args.sort != "none" and oldver is not None and newver is not None:
-          from .sortversion import sort_version_keys
-          version = sort_version_keys[args.sort]
-          if version(oldver) > version(newver): # type: ignore
-            arrow = f'{c.red}<-{c.normal}'
-            if args.newer:
-              continue
+        if version(oldver) > version(newver):  # type: ignore
+          if args.newer:
+            continue  # don't store this diff
+          diff['delta'] = 'old'
+        else:
+          diff['delta'] = 'new'
 
-        print(f'{name} {c.red}{oldver}{c.normal} {arrow} {c.green}{newver}{c.normal}')
+    elif oldver is None:
+      diff['delta'] = 'added'
+
+    elif newver is None:
+      if args.newer:
+        continue  # don't store this diff
+      diff['delta'] = 'gone'
+
+    if args.all or diff['delta'] != 'equal':
+      differences.append(diff)
+
+  if args.json:
+    if args.quiet:
+      print(json.dumps([diff['name'] for diff in differences], separators=(',', ':')))
+    else:
+      print(json.dumps(differences, sort_keys=True, separators=(',', ':')))
+
+  elif args.quiet:
+    for diff in differences:
+      print(diff['name'])
+
+  else:
+    from .lib.nicelogger import Colors, support_color
+    c = Colors(support_color(sys.stdout))
+
+    diffstyles = {
+      'new': {
+        'symbol': '->',
+        'oldc': c.red
+      },
+      'old': {
+        'symbol': f'{c.red}<-{c.normal}',
+        'oldc': c.red
+      },
+      'added': {
+        'symbol': '++',
+        'oldc': c.red
+      },
+      'gone': {
+        'symbol': f'{c.red}--{c.normal}',
+        'oldc': c.green
+      },
+      'equal': {
+        'symbol': '==',
+        'oldc': c.green
+      }
+    }
+
+    for diff in differences:
+      style = diffstyles[diff.get('delta', 'equal')] # type: ignore # mypy has issues with this line
+
+      print(f'{diff["name"]} {style["oldc"]}{diff["oldver"]}{c.normal} {style["symbol"]} {c.green}{diff["newver"]}{c.normal}')
